//	uart.h - include file for uart_xx..x utility functions
//	Mike Dawdy, Compuserve ID 76275,700

#include <conio.h>

//	1. INS8250-B and compatible UARTs

#define   TRB       0		// transmit and receive buffer
#define   DLL       0		// divisor latch least signif byte
#define   DLM       1		// divisor latch most signif byte
#define   IER       1		// interrupt enable register
#define   IIR       2		// interrupt identification register
#define   FCR       2		// FIFO control register
#define   LCR       3		// line control register
#define   MCR       4		// modem control register
#define   LSR       5		// line status register
#define   MSR       6		// modem status register
#define   SCR       7		// scratch register

#define   MCR_OUT2  0x08	// permit UART interrupts via MCR
#define   MCR_LOOP  0x10	// enable loopback mode via MCR

//	2. 8259A PIC: programmable interrupt controller

#define   PIC1_A0     0x20	// A0 address line for PIC 1
#define   PIC1_A1     0x21	// A1 address line for PIC 1
#define   PIC2_A0     0xa0	// A0 address line for PIC 2
#define   PIC2_A1     0xa1	// A1 address line for PIC 2
#define   PIC_IRR     0x0a	// prepare to read PIC IRR
#define   PIC_ISR     0x0b	// prepare to read PIC ISR
#define   PIC_EOI     0x20	// PIC auto end of interrupt command

//	3. 8253, 8254 PIT: programmable interval timer

#define   PIT_CWR     0x43	// control word register
#define   PIT_CTR2    0x42	// counter 2

//	4. other constant identifiers & macros

#define   IO_DELAY  _asm {jmp short $+2}
#define   TICKS     0x0000046c	// 0000:046c BIOS tick counter

//	5. function prototypes for utility functions

unsigned sys_config(void);

unsigned long uart_irq(unsigned addr);
unsigned uart_check(unsigned addr);
unsigned uart_type(unsigned addr);
void uart_restore(unsigned addr, unsigned char *ten_bytes);
void uart_registers(unsigned addr, unsigned char *ten_bytes);
void uart_reset(unsigned addr);
void uart_init(
	unsigned addr,unsigned baud,unsigned lcr,unsigned mcr,unsigned ier);
