//	uart_irq.c - determine IRQ level or IRQ conflict
//	Mike Dawdy, Compuserve ID 76275,700

// return 32 bit doubleword |ISR1|IRR1|ISR2|IRR2| 8 bits each field
// return value zero means timeout waiting for interrupt

#include      "uart.h"
#include      <dos.h>

#pragma       inline		// for Borland C: short and $
                                // requires Borland TASM
void _interrupt _far common_isr(void);
unsigned get_time(void);

static volatile unsigned char isr_1, irr_1, isr_2, irr_2;
static unsigned port, pic_2;

unsigned get_time(void)		// retrieve count from PIT counter 2
{
outp(PIT_CWR, 0x80);		// latch counter 2
IO_DELAY
return(inp(PIT_CTR2) | (inp(PIT_CTR2) << 8));
}


void _interrupt _far common_isr(void)
{
outp(PIC1_A0, PIC_ISR);        // prepare to read ISR of PIC 1
IO_DELAY                       // IBM AT Tech Ref page 9-8
isr_1 = inp(PIC1_A0);          // read ISR
IO_DELAY

outp(PIC1_A0, PIC_IRR);        // prepare to read IRR of PIC 1
IO_DELAY
irr_1 = inp(PIC1_A0);          // read IRR

if (pic_2 == 0x0040)
   {
   outp(PIC2_A0, PIC_ISR);	// prepare to read ISR of PIC 2
   IO_DELAY
   isr_2 = inp(PIC2_A0);	// read ISR
   IO_DELAY
   outp(PIC2_A0, PIC_IRR);	// prepare to read IRR of PIC 2
   IO_DELAY
   irr_2 = inp(PIC2_A0);	// read IRR
   }

inp(port+IIR);			// reset thr empty; then it re-arises
IO_DELAY			// IBM PS/2 & PC BIOS Tech Ref p. 4-19
if (isr_2 != 0)
   outp(PIC2_A0, PIC_EOI);	// reset ISR bit PIC 2
outp(PIC1_A0, PIC_EOI);		// reset ISR bit PIC 1
IO_DELAY
}

unsigned long uart_irq(unsigned addr)
{				// does not preserve UART registers
void (_interrupt _far *iv_save_1[8])(void);
void (_interrupt _far *iv_save_2[8])(void);

unsigned save_imr_1, save_imr_2, i;

pic_2 = sys_config() & 0xff40;	// pic_2 == 0x0040 means PIC 2 exists

save_imr_1 = inp(PIC1_A1);	// save state of PIC 1 IMR
if (pic_2 == 0x0040)
   save_imr_2 = inp(PIC2_A1);	// save state of PIC 2 IMR

_disable();                     // IBM PC & PS/2 BIOS Tech Ref P. 4-20
outp(PIC1_A1, 0xbc);		// mask (disable) IRQs 7, 5, 4, 3, 2
if (pic_2 == 0x0040)		// and, if PIC 2 exists, also
   outp(PIC2_A1, 0x9e);		// mask (disable) IRQs 15,12,11,10,9
_enable();
for (i = 1; i < 8; i++)		// while we save their interrupt
   if (i != 6)			// vectors and point them all to
      {				// common_isr()
      if (i != 1)
	 {
	 iv_save_1[i] = _dos_getvect(i+8);
	 _dos_setvect(i+8, common_isr);
	 }
      if ((pic_2 == 0x0040) && (i != 5))
         {
         iv_save_2[i] = _dos_getvect(i+0x70);
         _dos_setvect(i+0x70, common_isr);
         }
      }

uart_init(addr, 2400, 0x03, MCR_OUT2, 0x00);// needs IRQ-0 active

irr_1 = isr_1 = irr_2 = isr_2 = 0; // initialize global variables . .
port = addr;                    // for interrupt handler: common_isr()
				// init PIT counter-2 for ~1/18th sec
outp(PIT_CWR, 0xb4);		// counter-2,lsb/msb,mode 2,bin
IO_DELAY
outp(PIT_CTR2, 0x00);		// max initial count, LSB
IO_DELAY
outp(PIT_CTR2, 0x00);		// max initial count, MSB

_disable();
outp(PIC1_A1, 0x43);		// enable int's PIC 1 except 6, 1, 0
if (pic_2 == 0x0040)
   outp(PIC2_A1, 0x61);		// enable int's PIC 2 except 14,13,8
_enable();

outp(addr+IER, 0x02);           // generate an interrupt (THRE)
while ((isr_1 == 0) && (get_time() > 5877))
   ;				// wait max 1/20th sec for interrupt/timeout

uart_reset(addr);               // reset UART
outp(PIT_CWR, 0xb6);		// reset PIT counter-2 to BIOS spec

for (i = 1; i < 8; i++)		// restore saved int vectors
   if (i != 6)
      {
      if (i != 1)
	 _dos_setvect(i+8, iv_save_1[i]);
      if ((pic_2 == 0x0040) && (i != 5))
         _dos_setvect(i+0x70, iv_save_2[i]);
      }

_disable();
outp(PIC1_A1, save_imr_1);	// restore IMR of PIC 1
if (pic_2 == 0x0040)
   outp(PIC2_A1, save_imr_2);	// restore IMR of PIC 2
_enable();

return ((long)isr_1 << 24) | ((long)irr_1 << 16) | (isr_2 << 8) | irr_2;
}

